from .Ansi import ANSI


class console:
    @staticmethod
    def log(
        *values: object,
        sep: str | None = " ",
        end: str | None = f"{ANSI.RESET}\n",
    ):
        print(*values, sep=sep, end=end)

    @staticmethod
    def info(
        *values: object,
        sep: str | None = " ",
        end: str | None = f"{ANSI.RESET}\n",
    ):
        print(ANSI.CYAN, sep="", end="")
        print(*values, sep=sep, end=end)

    @staticmethod
    def error(
        *values: object,
        sep: str | None = " ",
        end: str | None = f"{ANSI.RESET}\n",
    ):
        print(ANSI.RED, sep="", end="")
        print(*values, sep=sep, end=end)

    @staticmethod
    def ex(
        value: Exception,
        sep: str | None = " ",
        end: str | None = f"{ANSI.RESET}\n",
    ):
        print(ANSI.RED, sep="", end="")
        print(f"[{type(value)}] {value}", sep=sep, end=end)

    @staticmethod
    def debug(
        *values: object,
        sep: str | None = " ",
        end: str | None = f"{ANSI.RESET}\n",
    ):
        print(ANSI.GREEN, sep="", end="")
        print(*values, sep=sep, end=end)



import sys


def get_raspberry_pi_model():
    # Apri il file /proc/cpuinfo in modalità di lettura
    with open('/proc/cpuinfo', 'r') as cpuinfo_file:
        # Leggi tutte le linee dal file
        lines = cpuinfo_file.readlines()

        # Cerca la linea che contiene il modello
        for line in lines:
            if line.startswith('Model'):
                # Estrai il modello dalla linea
                model_str = line.split(': ')[1].strip()
                return model_str

    # Se non è stata trovata la linea con il modello, restituisci None
    return None


if sys.implementation.name == "micropython":
    # Raspberry Pico Series
    print("Raspberry Pico Series")

    from .board_pico import Board
else:
    # Raspberry Model B | Raspberry Zero
    model = get_raspberry_pi_model()
    if model is None:
        raise Exception(f"Non è stato possibile recuperare il modello della Board.")

    if "Pi Zero" in model:
        print("Raspberry Pi Zero")

        from .board_zero import Board
    elif "Model B" in model:
        print("Raspberry Model B")

        from .board_model_b import Board
    else:
        raise Exception(f"Non è stato possibile trovare la Board per il modello: {model}.")




import os
import sys
import console


# -------------------------------------------------------------------------------------------- #
# Contenuto che dovrà essere scritto nel service file
CONTENT = """[Unit]
Description={name} service
After=network.target

[Service]
User=root
ExecStart=/bin/bash -c 'python3 {path}'
Restart=always
RuntimeMaxSec=infinity

[Install]
WantedBy=multi-user.target"""


# -------------------------------------------------------------------------------------------- #
def main() -> int:
    # Accetto solo 2 parametri:
    #   0. Nome del file che python eseguire
    #   1. Il mio parametro, ovvero il file che dovrà essere convertito in Service
    if len(sys.argv) != 2:
        raise Exception(
            "Fornire il path dello script che si vuole trasformare in Service!"
        )

    # Path Relativo -> MyProject/MyScript.py
    relative_script_path = sys.argv[1]
    # Path Absolute -> /home/<User>/Desktop/MyProject/MyScript.py
    abs_script_path = os.path.abspath(relative_script_path)
    # Path Dir -> /home/<User>/Desktop/MyProject
    dir_path = os.path.dirname(abs_script_path)
    # Full Name -> MyScript.py
    full_name_script_path = os.path.basename(relative_script_path)
    # Name -> MyScript
    name_script = ".".join(full_name_script_path.split(".")[:-1])

    # Check if Script is a File
    if not os.path.isfile(abs_script_path):
        raise Exception("Fornire il path di uno script .py!")

    # Creo un file momentaneo
    temp_file_path = os.path.join(dir_path, f"{name_script}.service")
    try:
        with open(temp_file_path, "w") as tmp_file:
            tmp_file.write(CONTENT.format(name=name_script, path=abs_script_path))

        # Controllo se il path finale esiste
        service_path = "/etc/systemd/system/"
        if not os.path.exists(service_path):
            raise Exception(f"Impossibile trovare la directory: '{service_path}'")

        # Copio il temp file nella folder dei servizi
        os.system(f"sudo cp {temp_file_path} {service_path}")
    finally:
        # Elimino il temp_file
        if os.path.exists(temp_file_path):
            os.remove(temp_file_path)

    # Controllo che mi trovo su Linux
    if sys.platform != "linux" and sys.platform != "linux2":
        raise Exception("Sistema operativo incompatibile, deve essere linux o linux2")

    # Enable Servizio & Start IT
    os.system("sudo systemctl daemon-reload")
    os.system(f"sudo systemctl enable {name_script}")
    os.system(f"sudo systemctl start {name_script}")
    os.system(f"sudo systemctl status {name_script}")

    return 0


if __name__ == "__main__":
    main()
